<?php
#----------------------------------------------------------------
#--Service Group Services Dashlet
#--Written by James Clark jclark@itconvergence.com
#----------------------------------------------------------------
#--Changes
#--     1.0.0   - 2014-12-03 - Initial 1.0 release
#--     1.5.0   - 2022-02-23 - Cleaned up code
#--                          - Added links
#--                          - Added colors
#--                          - Added last and next checks
#--	1.5.1	- 2022-02-25 - Added status text
#--			     - Changed Current State to words instead of Numbers
#--			     - Changed yellow color for WARNING
#--			     - Fixed display when no perfdata exists
#--	1.5.2	- 2022-02-25 - Sorted the list of servicegroups in the config popup
#--			     - Cleaned up config page layout
#--	1.5.3	- 2022-02-25 - Added SVG Name to top of table
#--	1.6.0	- 2022-03-02 - Added config option to show all services(default) or to show abbreviated display.
#--			       Added links to servicegroup commands
#--			       Added link to Show servicegroup details
#--			       Added count and links for status types
#--
#----------------------------------------------------------------
#-- NOTES:
#-- this function initialize the dashlet settings
#----------------------------------------------------------------
include_once(dirname(__FILE__).'/../dashlethelper.inc.php');
#----------------------------------------------------------------
#-- NOTES:
#-- this function initialize the dashlet settings
#----------------------------------------------------------------
servicegroupservices_dashlet_init();
#----------------------------------------------------------------   
#-- Description: This function initialize all the constants settings for the dashlet.
#-- NOTES:
#-- if you want to know all about the constant settings you can set go to COMPONENT DEFINITIONS, API Version inside constants.inc.php
#----------------------------------------------------------------
FUNCTION servicegroupservices_dashlet_init(){
	$name = "servicegroupservices";
	$args = 
	array(
		DASHLET_NAME => $name,
		DASHLET_VERSION => "1.6.0",
		DASHLET_DATE => "2022-03-02",
		DASHLET_AUTHOR => "ITconvergence",
		DASHLET_DESCRIPTION => "Service Group Services",					
		DASHLET_FUNCTION => "servicegroupservices_dashlet_func",
		DASHLET_TITLE => "Service Group Services",	
		DASHLET_OUTBOARD_CLASS => "servicegroupservices_outboardclass",
		DASHLET_INBOARD_CLASS => "servicegroupservices_inboardclass",
		DASHLET_PREVIEW_CLASS => "servicegroupservices_previewclass",		
		DASHLET_CSS_FILE => "css/servicegroupservices.css",  
		DASHLET_JS_FILE => "js/servicegroupservices.js", 
		DASHLET_WIDTH => "300px",
		DASHLET_HEIGHT => "300px",
		DASHLET_OPACITY => "1", 
		DASHLET_BACKGROUND => "",
		DASHLET_REFRESHRATE => 1,
	);
	register_dashlet($name,$args);
}
#----------------------------------------------------------------
#-- NOTES:
#-- This function work with a $mode, 4 modes exists  getconfightml, outboard,inboard,preview.
#-- getconfightml: is the first popup window when you add a dashlet to your dashboard.
#-- this window is used to configure your dashlet according your needs.
#---------------------------------------------------------------- 
FUNCTION servicegroupservices_dashlet_func($mode=DASHLET_MODE_PREVIEW,$id="",$args=null){
	$output = "";
	$imgbase = get_dashlet_url_base("servicegroupservices")."/images/";
	switch ($mode){
		case DASHLET_MODE_GETCONFIGHTML:
			$output .='<b>'.gettext('Service Group:').'</b><br/><select name="svg" id="svg"> ';
			$output .= servicegroupservices_get_servicegroups_option_list();
			$output .='</select><br><br>';
                        $output .='<b>Show Services</b><br/><select name="showservices" id="showservices"> ';
                        $output .='<option value="yes" selected>Yes, Show all services</option>';
                        $output .='<option value="no">No, show abbreviated list</option>';
                        $output .='</select><br><br>';
		break;
		case DASHLET_MODE_OUTBOARD:
		break;
		case DASHLET_MODE_INBOARD:    
			$backendargs = array();
			$svg = $args['svg'];
			$showservices = $args['showservices'];
			$id = "sgs_" . random_string(6);
			$output .= '<div class="svg" id="'.$id.'_dashlet">';
			$output .= '<input class="svg" type="hidden" name="'.$id.'_svg" value="'.$svg.'"/>';
			$output .= '<input class="showservices" type="hidden" name="'.$id.'_showservices" value="'.$showservices.'"/>';
			$output .= '<div id="'.$id.'_table">';
			$output .= servicegroupservices_get_service_option_list($svg,$showservices);
			$output .= '</div>';
			$output .= '</div><script type="text/javascript">set_interval_svg();</script>';
		break;
		case DASHLET_MODE_PREVIEW:
			$output = "<p><img src='".$imgbase."preview.png'></p>";
		break;
	}
	return $output;
}
#----------------------------------------------------------------
#-- This function queries all servicegroups and populates select
#-- list for the configuration
#----------------------------------------------------------------  
FUNCTION servicegroupservices_get_servicegroups_option_list(){
	$option_list = '';
	$option_list .='<option value="0" selected>Select group..</option>';
	$groups = get_xml_servicegroup_objects();
	$count = 1;
	foreach($groups as $data){
		$data =(array)$data;
		if ($count>1){
			#$option_list .='<option value="'.$data['servicegroup_name'].'">'.$data['servicegroup_name'].'</option>';			  
			$sortable[] = $data['servicegroup_name'];
		}
		$count = $count + 1;
	}
	sort($sortable);
	foreach($sortable as $data){
		$option_list .='<option value="'.$data.'">'.$data.'</option>';
	}
	return $option_list;
}
#----------------------------------------------------------------
#-- This function gets all services that are members of the
#-- configured servicegroup
#---------------------------------------------------------------- 
FUNCTION servicegroupservices_get_service_option_list($svg,$showservices){
	$table_draw=''; 
	$result_set =array();
	$backendargs = array();
	$backendargs['servicegroup_name'] = $svg;
	$members = get_xml_servicegroup_member_objects($backendargs);
	$table_draw .= '<table class="gridtable"><tr><th>Servicegroup</th><td colspan="300"><b>'.$svg.'</b></td></tr>';
	$table_draw .= "<tr><td colspan=300><b><a href=\"/nagiosxi/includes/components/xicore/status.php?show=services&servicegroup=".urlencode($svg)."\" style=\"color:#000000;\">ServiceGroup Details</a> - <a href=\"/nagiosxi/includes/components/nagioscore/ui/extinfo.php?type=8&servicegroup=".urlencode($svg)."\" style=\"color:#000000;\">ServiceGroup Commands</a></b></td></tr>";
	$table_draw .= '<tr><td colspan=300 style="background-color:#000000"></td></tr>';
	$ok=0;
	$warning=0;
	$critical=0;
	$unknown=0;
	foreach($members as $member){
		foreach($member as $data_source){
			foreach ($data_source as $key => $val){
				$val = (array)$val;
				if($showservices != "no"){
					$result_set = servicegroupservices_get_service_info($val['host_name'],$val['service_description'],$showservices);
					$table_draw.=$result_set;
				}else{
					$backendargs= array();
					$backendargs['service_description'] = $val['service_description'];
				        $backendargs['host_name']= $val['host_name'];
					$service_objects = get_xml_service_status($backendargs);
					$count++;	
				        foreach($service_objects as $service_element){
				                $service_element = (array) $service_element;
                				$currentstatus = $service_element['current_state'];
				                if($currentstatus == 0){
							$ok++;
				                } elseif($currentstatus == 1){
							$warning++;
				                } elseif($currentstatus == 2){
							$critical++;
				                } else{
							$unknown++;
				                }
					}
			
				}
			}
		}			
	}
	if($showservices == "no"){
	        $ok=$ok-$count;
		$okurl="<td style=\"background-color:#00FF00\"><a href=\"/nagiosxi/includes/components/xicore/status.php?&servicegroup=".urlencode($svg)."&show=services&hoststatustypes=15&servicestatustypes=2\" style=\"color:#000000;\"><b>OK - $ok</b></a></td>";
		$warningurl="<td style=\"background-color:#EDF246\"><a href=\"/nagiosxi/includes/components/xicore/status.php?&servicegroup=".urlencode($svg)."&show=services&hoststatustypes=15&servicestatustypes=4\" style=\"color:#000000;\"><b>WARNING - $warning</b></a></td>";
		$criticalurl="<td style=\"background-color:#FF5050\"><a href=\"/nagiosxi/includes/components/xicore/status.php?&servicegroup=".urlencode($svg)."&show=services&hoststatustypes=15&servicestatustypes=16\" style=\"color:#000000;\"><b>CRITICAL - $critical</b></a></td>";
		$unknownurl="<td style=\"background-color:#FF7F50\"><a href=\"/nagiosxi/includes/components/xicore/status.php?&servicegroup=".urlencode($svg)."&show=services&hoststatustypes=15&servicestatustypes=8\" style=\"color:#000000;\"><b>UNKNOWN - $unknown</b></a></td>";
		$table_draw.="<tr>$okurl$warningurl$criticalurl$unknownurl</tr>";
	}
	$table_draw .= '</table>';
	return $table_draw;
}
#----------------------------------------------------------------
#-- This function gets the information for individual services
#----------------------------------------------------------------   
FUNCTION servicegroupservices_get_service_info($host, $service_description,$showservices){
	$backendargs= array();
	$backendargs['service_description'] = $service_description;
	$backendargs['host_name']= $host;
	$service_objects = get_xml_service_status($backendargs);
	$table_header .= "<tr><th colspan=\"300\">".$host." - ".$service_description."</th></tr>";
        $table_header .= "";
	foreach($service_objects as $service_element){
     		$service_element = (array) $service_element;
                $lastcheck = $service_element['last_check'];
                $nextcheck = $service_element['next_check'];
                $statustext = $service_element['status_text'];
                $currentstatus = $service_element['current_state'];
                if($currentstatus == 0){
                        $color = "#00FF00";
			$status = "OK";
                } elseif($currentstatus == 1){
                        $color = "#EDF246";
			$status = "WARNING";
                } elseif($currentstatus == 2){
                        $color = "#FF5050";
			$status = "CRITICAL";
                } else{
                        $color = "#FF7F50";
			$status = "UNKNOWN";
                }
		if(count($service_element)>1){
			$table_header .= "<tr><th>Last Check</th><th>Next Check</th><th>Current Status</th></th>";
				$format = explode("=",str_replace(' ','=',$service_element['performance_data']));
				$table_data .= "<td>".$lastcheck."</td><td>".$nextcheck."</td><td style=\"background-color:".$color."\"><a href=\"/nagiosxi/includes/components/xicore/status.php?show=servicedetail&host=".urlencode($host)."&service=".urlencode($service_description)."\" style=\"color:#000000;\">".$status."</a></td>";
				for($i=0;$i<count($format);$i++){
					if($format[$i]!==""){
						if((($i+1)%2) !== 0){
							$table_header .= "<th>";
							$table_header .= $format[$i];
							$table_header .= "</th>";
						}else{
							$table_data .= "<td>";
							$pos = strpos($format[$i],';');
							if($pos === false){
								if( $format[$i] === "" || $format[$i] === null){
									$table_data .= '0';
								}else{
									$table_data .= $format[$i];
								}  
							}else{
								$token = explode(';',$format[$i]);
								if($token[0]==="" || $token[0] === null ){
									$table_data .= '0'; 
								}else{
									$table_data .= $token[0];
								}
							}
							$table_data .= "</td>";
						}
					} else {
						$table_header .= "<th colspan=\"300\">No Current Performace Data</th></tr>";
						$table_data .= "<td colspan=\"300\">There is no performance data</td>";
					}
				}
				$table_header .= "</tr>";	
			}
		}
	$table_data .= '</tr><tr><td colspan="300">'.$statustext.'</td></tr>';
	$result_set = $table_header.$table_data;	
	return $result_set;  
}
#----------------------------------------------------------------
#-- This function refreshes the dashlet
#---------------------------------------------------------------- 
FUNCTION servicegroupservices_refresh(){
	$request = $_REQUEST;
	$table_draw=''; 
        $table_draw.=servicegroupservices_get_service_option_list($request['svg']);
	return $table_draw;
}
?>
